/// CSS Custom Highlight API Module Level 1
///
/// https://drafts.csswg.org/css-highlight-api-1/

// ignore_for_file: unused_import

@JS('self')
@staticInterop
library css_highlight_api_1;

import 'dart:js_util' as js_util;
import 'package:js/js.dart';

import 'package:js_bindings/js_bindings.dart';

enum HighlightType {
  highlight('highlight'),
  spellingError('spelling-error'),
  grammarError('grammar-error');

  final String value;
  static HighlightType fromValue(String value) =>
      values.firstWhere((e) => e.value == value);
  static Iterable<HighlightType> fromValues(Iterable<String> values) =>
      values.map(fromValue);
  const HighlightType(this.value);
}

@JS()
@staticInterop
class Highlight extends JsIterable<AbstractRange> {
  external factory Highlight._(
      [AbstractRange? initialRanges1,
      AbstractRange? initialRanges2,
      AbstractRange? initialRanges3]);

  factory Highlight(
          [AbstractRange? initialRanges1,
          AbstractRange? initialRanges2,
          AbstractRange? initialRanges3]) =>
      Highlight._(
        initialRanges1 ?? undefined,
        initialRanges2 ?? undefined,
        initialRanges3 ?? undefined,
      );
}

extension PropsHighlight on Highlight {
  int get priority => js_util.getProperty(this, 'priority');
  set priority(int newValue) {
    js_util.setProperty(this, 'priority', newValue);
  }

  HighlightType get type =>
      HighlightType.fromValue(js_util.getProperty(this, 'type'));
  set type(HighlightType newValue) {
    js_util.setProperty(this, 'type', newValue.value);
  }
}

@JS()
@staticInterop
class HighlightRegistry extends JsMap<Highlight, String> {
  external factory HighlightRegistry();
}
