/// Web Animations Level 2
///
/// https://drafts.csswg.org/web-animations-2/

// ignore_for_file: unused_import

@JS('self')
@staticInterop
library web_animations_2;

import 'dart:js_util' as js_util;
import 'package:js/js.dart';
import 'package:meta/meta.dart';

import 'package:js_bindings/js_bindings.dart';

@JS()
@staticInterop
class GroupEffect {
  external factory GroupEffect._(Iterable<AnimationEffect> children,
      [dynamic timing]);

  factory GroupEffect(Iterable<AnimationEffect> children, [dynamic timing]) =>
      GroupEffect._(children, timing ?? undefined);
}

extension PropsGroupEffect on GroupEffect {
  AnimationNodeList get children => js_util.getProperty(this, 'children');
  AnimationEffect? get firstChild => js_util.getProperty(this, 'firstChild');
  AnimationEffect? get lastChild => js_util.getProperty(this, 'lastChild');
  dynamic clone() => js_util.callMethod(this, 'clone', []);

  void prepend(
          [AnimationEffect? effects1,
          AnimationEffect? effects2,
          AnimationEffect? effects3]) =>
      js_util.callMethod(this, 'prepend', [effects1, effects2, effects3]);

  void append(
          [AnimationEffect? effects1,
          AnimationEffect? effects2,
          AnimationEffect? effects3]) =>
      js_util.callMethod(this, 'append', [effects1, effects2, effects3]);
}

@JS()
@staticInterop
class AnimationNodeList {
  external factory AnimationNodeList();
}

extension PropsAnimationNodeList on AnimationNodeList {
  int get length => js_util.getProperty(this, 'length');
  AnimationEffect? item(int index) => js_util.callMethod(this, 'item', [index]);
}

@JS()
@staticInterop
class SequenceEffect implements GroupEffect {
  external factory SequenceEffect._(Iterable<AnimationEffect> children,
      [dynamic timing]);

  factory SequenceEffect(Iterable<AnimationEffect> children,
          [dynamic timing]) =>
      SequenceEffect._(children, timing ?? undefined);
}

extension PropsSequenceEffect on SequenceEffect {
  SequenceEffect clone() => js_util.callMethod(this, 'clone', []);
}

enum IterationCompositeOperation {
  replace('replace'),
  accumulate('accumulate');

  final String value;
  static IterationCompositeOperation fromValue(String value) =>
      values.firstWhere((e) => e.value == value);
  static Iterable<IterationCompositeOperation> fromValues(
          Iterable<String> values) =>
      values.map(fromValue);
  const IterationCompositeOperation(this.value);
}

///  Experimental: This is an experimental technologyCheck the
/// Browser compatibility table carefully before using this in
/// production.
///  The AnimationPlaybackEvent interface of the Web Animations API
/// represents animation events.
///  As animations play, they report changes to their [playState]
/// through animation events.
///
///
///
///    Event
///
///
///
///
///
///    AnimationPlaybackEvent
///
///
@experimental
@JS()
@staticInterop
class AnimationPlaybackEvent implements Event {
  external factory AnimationPlaybackEvent._(String type,
      [AnimationPlaybackEventInit? eventInitDict]);

  factory AnimationPlaybackEvent(String type,
          [AnimationPlaybackEventInit? eventInitDict]) =>
      AnimationPlaybackEvent._(type, eventInitDict ?? undefined);
}

extension PropsAnimationPlaybackEvent on AnimationPlaybackEvent {
  dynamic get currentTime => js_util.getProperty(this, 'currentTime');
  dynamic get timelineTime => js_util.getProperty(this, 'timelineTime');
}

@anonymous
@JS()
@staticInterop
class AnimationPlaybackEventInit implements EventInit {
  external factory AnimationPlaybackEventInit(
      {dynamic currentTime, dynamic timelineTime});
}

extension PropsAnimationPlaybackEventInit on AnimationPlaybackEventInit {
  dynamic get currentTime => js_util.getProperty(this, 'currentTime');
  set currentTime(dynamic newValue) {
    js_util.setProperty(this, 'currentTime', newValue);
  }

  dynamic get timelineTime => js_util.getProperty(this, 'timelineTime');
  set timelineTime(dynamic newValue) {
    js_util.setProperty(this, 'timelineTime', newValue);
  }
}
